
        .XCREF
        .NOLIST
        INCLUDE \masm32\include\masm32rt.inc
        .586
        INCLUDE \masm32\macros\timers.asm
        .LIST

;################################################################

;****************************************************************
; Equality Branches (Used for Signed or Unsigned Comparisons)
;----------------------------------------------------------------
; Instruction  Description               Condition       Aliases
;----------------------------------------------------------------
; JZ           Jump if equal             ZF=1            JE
; JNZ          Jump if not equal         ZF=0            JNE
;****************************************************************
;
;****************************************************************
; Unsigned Branches
;----------------------------------------------------------------
; Instruction  Description               Condition       Aliases
;----------------------------------------------------------------
; JA           Jump if above             CF=0 and ZF=0   JNBE
; JAE          Jump if above or equal    CF=0            JNC JNB
; JB           Jump if below             CF=1            JC JNAE
; JBE          Jump if below or equal    CF=1 or ZF=1    JNA
;****************************************************************
;
;****************************************************************
; Signed Branches
;----------------------------------------------------------------
; Instruction  Description               Condition       Aliases
;----------------------------------------------------------------
; JG           Jump if greater           SF=OF or ZF=0   JNLE
; JGE          Jump if greater or equal  SF=OF           JNL
; JL           Jump if less              SF<>OF          JNGE
; JLE          Jump if less or equal     SF<>OF or ZF=1  JNG
; JO           Jump if overflow          OF=1
; JNO          Jump if no overflow       OF=0
;****************************************************************

;################################################################

cmp128  PROTO   :DWORD,:DWORD
tstc128 PROTO   :DWORD,:DWORD
ShowCPU PROTO   :DWORD

;################################################################

        .DATA

pos1    dd 1,0,0,0
pos2    dd 2,0,0,0
posTest dd 0FFFFFFFEh,0FFFFFFFFh,0FFFFFFFFh,07FFFFFFFh
neg1    dd 0FFFFFFFFh,0FFFFFFFFh,0FFFFFFFFh,0FFFFFFFFh
neg2    dd 0FFFFFFFEh,0FFFFFFFFh,0FFFFFFFFh,0FFFFFFFFh
negTest dd 1,0,0,80000000h

NOstr   db 'NO ',0
OFstr   db 'OF ',0
NSstr   db 'NS ',0
SFstr   db 'SF ',0
NZstr   db 'NZ ',0
ZFstr   db 'ZF ',0
NCstr   db 'NC',13,10,0
CFstr   db 'CF'
CrLf    db 13,10,0

;################################################################

        .CODE

_main   PROC

        INVOKE  ShowCPU,0
        INVOKE  GetCurrentProcess
        INVOKE  SetProcessAffinityMask,eax,1
        INVOKE  Sleep,100
        print   offset CrLf
        INVOKE  tstc128,offset pos1,offset pos1    ;NO NS ZF NC
        INVOKE  tstc128,offset pos1,offset pos2    ;NO SF NZ CF
        INVOKE  tstc128,offset pos2,offset pos1    ;NO NS NZ NC
        INVOKE  tstc128,offset neg1,offset neg2    ;NO NS NZ NC
        INVOKE  tstc128,offset neg2,offset neg1    ;NO SF NZ CF
        INVOKE  tstc128,offset pos1,offset neg1    ;NO NS NZ CF
        INVOKE  tstc128,offset neg1,offset pos1    ;NO SF NZ NC
        INVOKE  tstc128,offset posTest,offset neg1 ;NO NS NZ CF
        INVOKE  tstc128,offset posTest,offset neg2 ;OF SF NZ CF
        INVOKE  tstc128,offset negTest,offset pos1 ;NO SF NZ NC
        INVOKE  tstc128,offset negTest,offset pos2 ;OF NS NZ NC
        print   offset CrLf
        mov     ecx,5

Ctime0: push    ecx

        counter_begin 25000000,HIGH_PRIORITY_CLASS

        INVOKE  cmp128,offset pos1,offset pos2

        counter_end

        print   ustr$(eax),32
        pop     ecx
        dec     ecx
        jnz     Ctime0

        print   offset CrLf
        print   offset CrLf
        inkey
        exit

_main   ENDP

;****************************************************************

tstc128 PROC    parm1:DWORD,parm2:DWORD

        push    ebx
        INVOKE  cmp128,parm1,parm2
        pushfd
        pop     ebx
        test    bh,8                   ;overflow flag
        mov     eax,offset NOstr
        jz      ts128A

        mov     eax,offset OFstr

ts128A: print   eax
        test    bl,80h                 ;sign flag
        mov     eax,offset NSstr
        jz      ts128B

        mov     eax,offset SFstr

ts128B: print   eax
        test    bl,40h                 ;zero flag
        mov     eax,offset NZstr
        jz      ts128C

        mov     eax,offset ZFstr

ts128C: print   eax
        shr     ebx,1                  ;carry flag
        mov     eax,offset NCstr
        jnc     ts128D

        mov     eax,offset CFstr

ts128D: print   eax
        pop     ebx
        ret

tstc128 ENDP

;****************************************************************

        OPTION  PROLOGUE:None
        OPTION  EPILOGUE:None

cmp128  PROC    lpNum1:DWORD,lpNum2:DWORD

;Compare 2 128-bit values
;written by Dave Sheldon, 5-2011

;could easily be expanded to handle larger values or variable sizes

        push    ebx
        mov     edx,[esp+8]            ;lpNum1
        mov     ebx,[esp+12]           ;lpNum2
        mov     eax,[edx]
        mov     ecx,[edx+4]
        sub     eax,[ebx]
        sbb     ecx,[ebx+4]
        push    eax
        push    ecx
        mov     eax,[edx+8]
        mov     ecx,[edx+12]
        sbb     eax,[ebx+8]
        sbb     ecx,[ebx+12]
        pop     edx
        pop     ebx
        pushfd
        or      eax,ecx
        or      eax,ebx
        pop     ecx
        or      eax,edx
        lahf
        pop     ebx
        and     ah,40h
        and     cx,8BFh
        or      ah,cl
        add     ch,78h
        sahf
        ret     8

cmp128  ENDP

        OPTION  PROLOGUE:PrologueDef
        OPTION  EPILOGUE:EpilogueDef

;****************************************************************

OPTION PROLOGUE:NONE
OPTION EPILOGUE:NONE
ShowCPU proc mode:DWORD

;ShowCPU written by Jochen (jj2007)

; Usage:

; call ShowCPU for the short,

; push -127
; call ShowCPU for the long version

  push ebx
  ifndef MyCPU
	.data?
	CpuFlags		dd ?
	CpuExFlags	dd ?
	CpuMMX	dd ?
	CpuSSE	dd ?
	CpuFxSR	dd ?
	CpuManu	dd 20 dup (?)
	CpuInfo	dd 100 dup (?)
	stepping	dd ?
	model	dd ?
	family	dd ?
	ptype	dd ?
	extmodel	dd ?
	extfamily	dd ?
	.data
	txStep	db "Step           ", 9, 0
	txModel	db 13, 10, "Model        ", 9, 0
	txFam	db 13, 10, "Family       ", 9, 0
	txType	db 13, 10, "Type          ", 9, 0
	txExtM	db 13, 10, "Ext model   ", 9, 0
	txExtF	db 13, 10, "Ext family   ", 9, 0
	txCpuMan	db 13, 10, "Manufacturer", 9, 0
	txP1	db "P1 (1993+)", 0
	txPro	db "Pentium Pro (1995+)", 0
	txP2	db "P2 (1997+)", 0
	txP3	db "P3 (2000+)", 0
	txP4	db "P4 (2000+)", 0
	txIta1	db "Itanium 1 (2001+)", 0
	txIta2	db "Itanium 2 (2002+)", 0
	txPM	db "Pentium M (2003+)", 0
	txCore	db "Core (2006+)", 0
	txCore2	db "Core Duo (2006+)", 0
	txP4Prescott	db "Pentium 4 Prescott (2005+)", 0
	txUnknownP4	db "Pentium 4 (?)", 0
	txUnknown	db "?", 0
	.code
  endif
  xor eax, eax
  db 0Fh, 0A2h		; cpuid 0
  mov CpuManu, ebx
  mov CpuManu[4], edx
  mov CpuManu[8], ecx
  push 1
  pop eax
  db 0Fh, 0A2h	; 					; cpuid 1
  mov CpuFlags, edx
  mov CpuExFlags, ecx
  and CpuMMX, 0
  and CpuSSE, 0

  mov ebx, eax
  and eax, 15
  mov stepping, eax

  shr ebx, 4
  mov eax, ebx
  and eax, 15
  mov model, eax

  shr ebx, 4
  mov eax, ebx
  and eax, 15
  mov family, eax

  shr ebx, 4
  mov eax, ebx
  and eax, 15
  mov ptype, eax

  shr ebx, 4
  mov eax, ebx
  and eax, 15
  mov extmodel, eax

  shr ebx, 4
  mov eax, ebx
  and eax, 15
  mov extfamily, eax

  invoke lstrcpy, offset CpuInfo, offset txStep
  invoke lstrcat, offset CpuInfo, str$(stepping)
  
  invoke lstrcat, offset CpuInfo, offset txModel
  invoke lstrcat, offset CpuInfo, str$(model)

  invoke lstrcat, offset CpuInfo, offset txFam
  invoke lstrcat, offset CpuInfo, str$(family)

  invoke lstrcat, offset CpuInfo, offset txType
  invoke lstrcat, offset CpuInfo, str$(ptype)

  invoke lstrcat, offset CpuInfo, offset txExtM
  invoke lstrcat, offset CpuInfo, str$(extmodel)

  invoke lstrcat, offset CpuInfo, offset txExtF
  invoke lstrcat, offset CpuInfo, str$(extfamily)

  invoke lstrcat, offset CpuInfo, offset txCpuMan
  invoke lstrcat, offset CpuInfo, offset CpuManu

  mov eax, [esp+8]
  .if eax==-127
	print "CPU family ",
	print str$(family), ", model "
	print str$(model), ", "
  .endif
  
  COMMENT @
[url=http://www.pagetable.com/?p=18]CPUID Family codes[/url]
* 486 (1989): family 4
* Pentium (1993): family 5
* Pentium Pro (1995): family 6, models 0 and 1
* Pentium 2 (1997): family 6, models 3, 5 and 6
* Pentium 3 (2000): family 6, models 7, 8, 10, 11
* Itanium (2001): family 7
* Pentium 4 (2000): family 15/0
* Itanium 2 (2002): family 15/1 and 15/2
* Pentium M (2003): family 6, models 9 and 13
* Core (2006): family 6, model 14
* Core 2 (2006): family 6, model 15 @
  .if family==5
	mov edx, offset txP1
  .elseif family==6
	.if model<=1
		mov edx, offset txPro
	.elseif model==3 || model==5 || model==6
		mov edx, offset txP2
	.elseif model==7 || model==8 || model==10 || model==11
		mov edx, offset txP3
	.elseif model==9 || model==13
		mov edx, offset txPM
	.elseif model==14
		mov edx, offset txCore
	.elseif model==15
		mov edx, offset txCore2
	.endif
  .elseif family==7	; Itanium
		mov edx, offset txIta1
  .elseif family==15
	.if model==0
		mov edx, offset txP4
	.elseif model<=2
		mov edx, offset txIta2
	.elseif model==4
		mov edx, offset txP4Prescott
	.else
		mov edx, offset txUnknownP4
	.endif
  .else
	mov edx, offset txUnknown
  .endif
  print edx
  mov edx, CpuFlags
  ; MsgBox 0, BIN$(edx, f), "Hi", MB_OK
  bt edx, 23
  .if Carry?
	inc CpuMMX
	print ", MMX, SSE"
  .else
	print ", no MMX, SSE"
  .endif

  mov edx, CpuFlags
  bt edx, 24
  jnc @F
  inc CpuFxSR	; supports FXSAVE, FXRSTOR
@@:

  bt edx, 25
  jnc @F
  inc CpuSSE
@@:

  bt edx, 26
  jnc @F
  inc CpuSSE
@@:

  mov ecx, CpuExFlags
  bt ecx, 0
  jnc @F
  inc CpuSSE
@@:
  print str$(CpuSSE), 13, 10
  pop ebx
  mov eax, [esp+4]
  .if eax==-127
	print offset CpuInfo, 13, 10
	ret 4
  .else
	ret
  .endif
ShowCPU endp
OPTION PROLOGUE:PrologueDef 
OPTION EPILOGUE:EpilogueDef

;################################################################

        END     _main

